<?php

namespace Drupal\inactive_autologout\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Inactive logout settings form.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity type manager service.
   *
   * @var \Drupal\user\Entity\Role
   */
  protected $userRoleStorage;

  /**
   * Constructs a SettingsForm object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   Entity type manager service.
   */
  public function __construct(ConfigFactoryInterface $config_factory, EntityTypeManagerInterface $entity_type_manager) {
    parent::__construct($config_factory);
    $this->entityTypeManager = $entity_type_manager;
    $this->userRoleStorage = $this->entityTypeManager->getStorage('user_role');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'inactive_autologout.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'inactive_autologout_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('inactive_autologout.settings');

    $form['autologout'] = [
      '#type' => 'details',
      '#open' => TRUE,
      '#title' => $this->t("Auto Logout"),
    ];

    $form['autologout']['enable'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable'),
      '#default_value' => ($config->get('enable') !== NULL) ? $config->get('enable') : FALSE,
    ];

    $defaults = 120;
    if (!empty($config->get('timeout'))) {
      $defaults = $config->get('timeout');
    }

    $form['autologout']['timeout'] = [
      '#type' => 'number',
      '#title' => $this->t('Timeout value in seconds'),
      '#description' => $this->t('The length of inactivity time, in seconds, before automated log out. Must be 120 seconds or greater. Will not be used if role timeout is activated.'),
      '#default_value' => $defaults,
      '#required' => TRUE,
    ];

    $roles = $this->userRoleStorage->loadMultiple();
    $options = [];
    foreach ($roles as $role) {
      $options[$role->id()] = $role->label();
    }
    unset($options['anonymous']);
    unset($options['authenticated']);

    $form['autologout']['roles'] = [
      '#type' => 'checkboxes',
      '#multiple' => TRUE,
      '#options' => $options,
      '#title' => $this->t("Roles"),
      '#description' => $this->t('Check roles for enable auto logout.'),
      '#default_value' => (!empty($config->get('roles'))) ? $config->get('roles') : [],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $timeout = $form_state->getValue('timeout');
    if ($timeout < 120) {
      $form_state->setErrorByName('timeout', $this->t('Must be 120 or greater.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);

    $this->config('inactive_autologout.settings')
      ->set('timeout', $form_state->getValue('timeout'))
      ->set('roles', $form_state->getValue('roles'))
      ->set('enable', $form_state->getValue('enable'))
      ->save();

  }

}
